extends MarginContainer
class_name ProfTest
var msec_t = 1
var subclass
var subnative
var engine
var vec
var typed_vec := Vector3(1,1,1)
var world2d
var selected_func = "NONE"
var rayquery = PhysicsRayQueryParameters2D.create(Vector2(0, 0), Vector2(50, 100))
var rayquery_typed := PhysicsRayQueryParameters2D.create(Vector2(0, 0), Vector2(50, 100))

const CALL_TYPES = {
	"NONE": "Do nothing",
	"IDENT_COMMON": "Normal self function call in the same class.",
	"IDENT_COMMON_MANY_CALLS": "As above, repeated <it> times",
	"IDENT_STATIC": "Normal static function call in the same class.",
	"IDENT_STATIC_NESTED": "2 nested static function calls in the same class.",
	"IDENT_EXTENDED_FUNC": "Native function of the class ancestor. find_child",
	"IDENT_NEW": "Bare new().",
	"IDENT_COMMON_AWAITED": "Awaited self function call in the same class.",
	"IDENT_STATIC_AWAITED": "Awaited static function call in the same class.",
	"IDENT_TYPED": "Normal self function call, typed",
	"SUB_CLASS_STATIC": "Static call on a generic RefCounted class from the same script",
	"SUB_CLASS_NESTEDSTATIC": "2 nested static calls on a generic RefCounted class",
	"SUB_CLASS_METHOD": "Normal Method",
	"SUB_CLASS_METHOD_call": "Normal method called with call()",
	"SUB_CLASS_NATIVE_call": "Native method called with call()",
	"SUB_CLASS_METHOD_call_deferred": "Normal method called with deferred_call()",
	"SUB_CLASS_NATIVE_call_deferred": "Native method called with deferred_call()",
	"SUB_PHYSICSPROC_NATIVE": "Native call within _physics_process, from a normal untyped variable.\nPhysicsDirectSpaceState2D.intersect_ray()",
	"SUB_COMMON_NATIVE": "Normal native call from untyped var.",
	"SUB_COMMON_AUDIOSERVER": "Audioserver native call",
	"SUB_COMMON_RAYCAST": "Raycast call",
	"SUB_AWAITED_NATIVE": "Awaited native call from untyped var.",
	"SUB_TYPED_NATIVE": "Native call from typed var",
	"SUB_STATIC_NATIVE": "Static native call",
	"SUB_BUILTIN_STATIC": "Static of a builtin type",
	"SUB_BUILTIN_TYPED": "Fully typed builtin call. vector3.dot",
	"SUB_BUILTIN_NONTYPED": "Common builtin call. vector3.dot"
}



class SomeClass:
	extends Node

	static func static_method():
		for i in 10000:
			i * i

	func normal_method():
		for i in 10000:
			i * i
		
		
	static func nesting_static_method():
		for i in 10000:
			i * i
		
		nested_static_method()

	static func nested_static_method():
		for i in 10000:
			i * i
		

func _ready():
	subclass = SomeClass.new()
	subnative = RandomNumberGenerator.new()
	engine = Engine.get_singleton("Engine")
	vec = Vector3(1,2,3)
	world2d = get_world_2d()

	var button = find_child("OptionButton")
	var i = 0
	for calltype in CALL_TYPES:
		button.add_item(calltype, i)
		button.set_item_tooltip(i, CALL_TYPES[calltype])
		i += 1
	button.item_selected.connect(func_changed)
	button.select(CALL_TYPES.keys().find(selected_func))
	_on_h_slider_value_changed(msec_t)

func func_changed(index):
	var button = find_child("OptionButton")
	selected_func = button.get_item_text(index)

static func static_func():
	for i in 10000:
		i * i
	

func normal_func_single():
	for i in 10000:
		i * i
	
func normal_func():
	for i in 10000:
		i * i
	

func typed_func(a: int) -> void:
	for i in 10000:
		i * i
	

static func nesting_static_func():
	for i in 10000:
		i * i
	
	nested_static_func()

static func nested_static_func():
	for i in 10000:
		i * i
	

func _physics_process(delta):
	if selected_func == "SUB_PHYSICSPROC_NATIVE":
		var start = Time.get_ticks_msec()
		while Time.get_ticks_msec() - start < msec_t:
			var result = world2d.direct_space_state.intersect_ray(rayquery)

func _process(delta):
	var start = Time.get_ticks_msec()
	while Time.get_ticks_msec() - start < msec_t:
		match selected_func:

			"IDENT_COMMON":
				normal_func()
			"IDENT_STATIC":
				static_func()
			"IDENT_STATIC_NESTED":
				nesting_static_func()
			"IDENT_EXTENDED_FUNC":
				find_child("asdf")

			"IDENT_NEW":
				var a = new()
				a.queue_free()
			"IDENT_COMMON_AWAITED":
				await normal_func()
			"IDENT_STATIC_AWAITED":
				await nested_static_func()
			"IDENT_TYPED":
				var a := 5
				typed_func(a)
			"SUB_CLASS_STATIC":
				SomeClass.static_method()
			"SUB_CLASS_NESTEDSTATIC":
				SomeClass.nesting_static_method()
			"SUB_CLASS_METHOD":
				subclass.normal_method()
			"SUB_CLASS_METHOD_call":
				subclass.call("normal_method", )
			"SUB_CLASS_NATIVE_call":
				subclass.call("find_child","asdf")
			"SUB_CLASS_METHOD_call_deferred":
				subclass.call_deferred("normal_method")
			"SUB_CLASS_NATIVE_call_deferred":
				subclass.call_deferred("find_child","asdf")
			"SUB_PHYSICSPROC_NATIVE":
				break
			"SUB_COMMON_NATIVE":
				find_children("asdf")
			"SUB_COMMON_AUDIOSERVER":
				AudioServer.get_input_device_list()
			"SUB_COMMON_RAYCAST":
				world2d.direct_space_state.intersect_ray(rayquery)
			"SUB_AWAITED_NATIVE":
				await find_children("asdf")
			"SUB_TYPED_NATIVE":
				world2d.direct_space_state.intersect_ray(rayquery_typed)
			"SUB_STATIC_NATIVE":
				pass
			"SUB_BUILTIN_STATIC_TYPED":
				Basis.looking_at(typed_vec)
			"SUB_BUILTIN_STATIC":
				Basis.looking_at(vec)
			"SUB_BUILTIN_TYPED":
				typed_vec.dot(typed_vec)
			"SUB_BUILTIN_NONTYPED":
				vec.dot(vec)
			_:
				pass



func _on_h_slider_value_changed(value):
	msec_t = int(value)
	find_child("Label").set_text("msecs: " + str(msec_t))
